/*
    DB Port
    Copyright (C) 2010 Daniel Mewes
*/

package dbport;

import java.util.*;
import java.net.*;
import java.io.*;

class ConnectionHandler extends Thread {
    private Socket connection;
    private DbEnvironment environment;
    private InputStream input;
    private OutputStream output;
    
    
    public ConnectionHandler(Socket connection, DbEnvironment environment) throws IOException {
        this.connection = connection;
        this.environment = environment;
        
        input = connection.getInputStream();
        output = connection.getOutputStream();
    }
    
    public void run() {
        // Lock database exclusively... (if your DB has its own concurrency control, you may consider changing this)
        environment.dbMutex.lock();
    
        try {
            HttpHandler handler = new HttpHandler(input, output);
            handler.readRequest();
            
            String page = handler.getRequestPage();
            
            System.err.println("Got request: " + handler.getRequestUrl());
            
            handler.setContentType("text/html; charset=utf-8");
            handler.sendHeader();    
            
            try {
                // Serve request with appropriate page
            
                if (page.equals("/"))
                    handler.sendContent(StartPage.getContent(environment));
                else if (page.equals("/shutdown")) {
                    handler.sendContent(ShutdownPage.getContent(environment));
                    environment.listening = false; // Does not really work atm...
                }
                else if (page.equals("/kill")) {
                    handler.sendContent(KillPage.getContent(environment));
                    environment.listening = false; // Does not really work atm...
                }
                else if (page.equals("/restart"))
                    handler.sendContent(RestartPage.getContent(environment));
                else if (page.equals("/listTables"))
                    handler.sendContent(ListTablesPage.getContent(environment));
                else if (page.equals("/createTable"))
                    handler.sendContent(CreateTablePage.getContent(environment, handler.getRequestPageArguments()));
                else if (page.equals("/dropTable"))
                    handler.sendContent(DropTablePage.getContent(environment, handler.getRequestPageArguments()));
                else if (page.equals("/createIndex"))
                    handler.sendContent(CreateIndexPage.getContent(environment, handler.getRequestPageArguments()));
                else if (page.equals("/dropIndex"))
                    handler.sendContent(DropIndexPage.getContent(environment, handler.getRequestPageArguments()));
                else if (page.equals("/describeIndexes"))
                    handler.sendContent(DescribeIndexesPage.getContent(environment));
                else if (page.equals("/insertData"))
                    handler.sendContent(InsertDataPage.getContent(environment, handler.getRequestPageArguments()));
                else if (page.equals("/queryData")) // TODO: Support way for updating data
                    handler.sendContent(QueryDataPage.getContent(environment, handler.getRequestPageArguments()));
                else
                    handler.sendContent(ErrorPage.getContent(environment));
            }
            catch (Exception e) {
                handler.sendContent(MessagePage.getContent(environment, "Exception: " + e.toString()));
            }            
        } catch (Exception e) {
            System.err.println("Got exception while serving connection: " + e);
        }
        
        // Unlock database
        environment.dbMutex.unlock();
        
        try {
            connection.close();
        } catch (IOException e) {}
    }
}
