/*
    DB Port
    Copyright (C) 2010 Daniel Mewes
*/

package dbport;

import java.util.*;
import java.net.*;
import java.io.*;

class HttpHandler {
    private InputStream input;
    private OutputStream output;
    
    private String contentType;
    private String requestUrl;
    private String requestPage;
    private String[] requestPageArguments;
    private String requestHost;
    
    
    public HttpHandler(InputStream input, OutputStream output) {
        this.input = input;
        this.output = output;
    }
    
    public void readRequest() throws IOException, NumberFormatException {
        while (true) {
            String line = readLine();
            if (line.length() == 0)
                return;
            
            String[] words = line.split(" ");
            
            if (words.length > 0) {
                if (words[0].equals("GET") && words.length > 1) {
                    requestUrl = words[1];
                    String[] pageArgs = requestUrl.split("\\?", 2);
                    requestPage = pageArgs[0];
                    if (pageArgs.length > 1) {
                        pageArgs[1] = pageArgs[1].replace('+', ' ');
                        int pos = -1;
                        while ((pos = pageArgs[1].indexOf("%")) >= 0 && pageArgs[1].length() >= pos + 3) {
                            String hexString = pageArgs[1].substring(pos + 1, pos + 3);
                            int decodedChar = Integer.decode("0x" + hexString).intValue();
                            pageArgs[1] = pageArgs[1].substring(0, pos) + (char)decodedChar + pageArgs[1].substring(pos + 3);
                        }
                        requestPageArguments = pageArgs[1].split("&");
                    }
                    else
                        requestPageArguments = new String[0];
                }
                else if (words[0].equals("Host:") && words.length > 1)
                    requestHost = words[1];
            }
        }
    }
    
    public String getRequestUrl() {
        return requestUrl;
    }
    
    public String getRequestPage() {
        return requestPage;
    }
    
    public String[] getRequestPageArguments() {
        return requestPageArguments;
    }
    
    public String getRequestHost() {
        return requestHost;
    }
    
    public void setContentType(String contentType) {
        this.contentType = contentType;
    }
    
    public void sendHeader() throws IOException {
        writeLine("HTTP/1.0 200 OK");
        writeLine("Connection: close");
        writeLine("Cache-Control: no-cache, must-revalidate");
        writeLine("Pragma: no-cache");

        if (contentType != null)
            writeLine("Content-Type: " + contentType);
            
        writeLine("");
    }
    
    public void sendContent(String content) throws IOException {
        writeString(content);
    }
    
    private String readLine() throws IOException {
        StringBuffer result = new StringBuffer();
        int ch = 0;
        while ((ch = input.read()) != -1) {
            if ((char)ch == '\r'); // Ignore...
            else if ((char)ch == '\n')
                return result.toString();
            else
                result.append((char)ch);
        }
        
        return result.toString();
    }
    
    private void writeLine(String line) throws IOException {
        writeString(line + "\r\n");
    }
    
    private void writeString(String str) throws IOException {
        output.write(str.getBytes());
    }
    
}
