/*
    DB Port
    Copyright (C) 2010 CMS (Carl, Mewes, Scheweiler Wuerzburger)
*/


package dbport;

import exceptions.IsLeafException;
import exceptions.NotLeafNodeException;
import systeminterface.PredicateTreeNode;
import util.ComparisonOperator;
import util.LogicalOperator;

public class PrimitivePredicateTreeNode implements PredicateTreeNode {

	private PredicateTreeNode leftChild;
	private LogicalOperator logicalOperator;
	private PredicateTreeNode rightChild;
	
	private String column;
	private ComparisonOperator comparisonOperator;
	private Object value;
	
	public PrimitivePredicateTreeNode(PredicateTreeNode leftChild, LogicalOperator logicalOperator, 
			PredicateTreeNode rightChild) {
		this.leftChild = leftChild;
		this.rightChild = rightChild;
		this.logicalOperator = logicalOperator;
		column = null;
		comparisonOperator = null;
		value = null;
	}
	
	public PrimitivePredicateTreeNode(String column, ComparisonOperator comparisonOperator, 
			Object value) {
		this.column = column;
		this.comparisonOperator = comparisonOperator;
		this.value = value;
		logicalOperator = null;
		leftChild = null;
		rightChild = null;
	}

	@Override
	public String getColumnName() throws NotLeafNodeException {
		if (!isLeaf())
			throw new NotLeafNodeException();
		return column;
	}
	
	@Override
	public ComparisonOperator getComparisonOperator()
			throws NotLeafNodeException {
		if (!isLeaf())
			throw new NotLeafNodeException();
		return comparisonOperator;
	}

	@Override
	public PredicateTreeNode getLeftChild() throws IsLeafException {
		if (isLeaf())
			throw new IsLeafException();
		return leftChild;
		
	}

	@Override
	public LogicalOperator getLogicalOperator() throws IsLeafException {
		if (isLeaf())
			throw new IsLeafException();
		return logicalOperator;
	}

	@Override
	public PredicateTreeNode getRightChild() throws IsLeafException {
		if (isLeaf())
			throw new IsLeafException();
		return rightChild;
	}

	@Override
	public Object getValue() throws NotLeafNodeException {
		if (!isLeaf())
			throw new NotLeafNodeException();
		return value;
	}

	@Override
	public boolean isLeaf() {
		return logicalOperator == null;
	}
	
}
