/*
    DB Port
    Copyright (C) 2010 CMS (Carl, Mewes, Scheweiler Wuerzburger)
*/

package dbport;

import java.util.*;
import metadata.*;
import exceptions.NoSuchColumnException;
import systeminterface.Row;

/*
    This is a basic row which can be used to construct and insert rows into a table
*/

public class PrimitiveRow implements Row {
	
	private Map<String, Type> schema;
	private Map<String, Object> data;
	
	public PrimitiveRow(Map<String, Type> schema) {
		this.schema = schema;
        this.data = new HashMap<String, Object>();
	}

	@Override
	public int getColumnCount() {
		return schema.size();
	}

	@Override
	public Type getColumnType(String columnName) throws NoSuchColumnException {
		if (schema.containsKey(columnName))
		    return schema.get(columnName);
		else
		    throw new NoSuchColumnException();
	}

	@Override
	public Object getColumnValue(String columnName)
			throws NoSuchColumnException {
		if (schema.containsKey(columnName)) {
		    if (data.containsKey(columnName))
		        return data.get(columnName);
		    else
		        return null;
		}
		else
		    throw new NoSuchColumnException();
	}
	
	public void setColumnValue(String columnName, Object value) throws NoSuchColumnException {
	    if (schema.containsKey(columnName))
    	    data.put(columnName, value);
    	else
    	    throw new NoSuchColumnException();
	}
	
	@Override
	public String[] getColumnNames() {
	    return (String[])schema.keySet().toArray(new String[0]);
	}	
}
